import logging

from DateTime import DateTime
from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
from Products.CMFCore.utils import getToolByName

from Products.plonehrm import utils
from Products.plonehrm.controlpanel import IHRMNotificationsPanelSchema
from plonehrm.contracts import ContractsMessageFactory as _
from plonehrm.contracts.notifications.events import ContractEndingEvent
from plonehrm.contracts.notifications.events import TrialPeriodEndingEvent
from plonehrm.notifications.emailer import HRMEmailer
from plonehrm.notifications.interfaces import INotified
from plonehrm.notifications.utils import get_employees_for_checking
from zope.event import notify

logger = logging.getLogger("plonehrm.contracts:")

EXPIRY_NOTIFICATION = u"plonehrm.contracts: Expiry notification"
TRIAL_PERIOD_ENDING_NOTIFICATION = u"plonehrm.contracts: Trial period ending"


def contract_ending_checker(object, event):
    """Check if the current contract of employees is almost ending.

    object is likely the portal, but try not to depend on that.
    """
    portal = getToolByName(object, 'portal_url').getPortalObject()
    panel = IHRMNotificationsPanelSchema(object)
    if not panel.contract_expiry_notification:
        logger.info("Contract ending notification is switched off.")
        return
    days_warning = panel.contract_expiry_notification_period

    employees = get_employees_for_checking(object)
    for brain in employees:
        try:
            employee = brain.getObject()
        except (AttributeError, KeyError):
            logger.warn("Error getting object at %s", brain.getURL())
            continue
        contracts = employee.restrictedTraverse('@@contracts')
        current_contract = contracts.current_contract()
        info = dict(employee_name = employee.officialName(),
                    employee_url = employee.absolute_url())
        if current_contract is None:
            #email = HRMEmailer(employee)
            #email.template = ZopeTwoPageTemplateFile('no_current_contract.pt')
            #email.send()
            #notify(NoContractEvent(employee))
            continue
        expires = contracts.expires()
        if expires is None:
            continue
        now = DateTime()
        if now + days_warning >= expires:
            # Check if we have already warned about this.
            notified = INotified(current_contract)
            if notified.has_notification(EXPIRY_NOTIFICATION):
                continue
            info['days'] = days_warning
            options = dict(days=days_warning)
            template = ZopeTwoPageTemplateFile('contract_nears_ending.pt')
            addresses = utils.email_adresses_of_local_managers(employee)
            recipients = (addresses['worklocation_managers'] +
                          addresses['hrm_managers'])
            email = HRMEmailer(employee,
                               template=template,
                               options=options,
                               recipients=recipients,
                               subject=_(u'A contract nears ending'))
            email.send()
            notify(ContractEndingEvent(employee))
            notified.add_notification(EXPIRY_NOTIFICATION)


def trial_period_ending_checker(object, event):
    """Check if the trial period of employees is almost ending.

    object is likely the portal, but try not to depend on that.
    """
    portal = getToolByName(object, 'portal_url').getPortalObject()
    panel = IHRMNotificationsPanelSchema(object)
    if not panel.trial_ending_notification:
        logger.info("Trial period ending notification is switched off.")
        return
    days_warning = panel.trial_ending_notification_period

    employees = get_employees_for_checking(portal)
    for brain in employees:
        try:
            employee = brain.getObject()
        except (AttributeError, KeyError):
            logger.warn("Error getting object at %s", brain.getURL())
            continue
        contracts = employee.restrictedTraverse('@@contracts')
        current_contract = contracts.current_contract()
        info = dict(employee_name = employee.officialName(),
                    employee_url = employee.absolute_url())
        if current_contract is None:
            continue
        trial_period_end = contracts.trial_period_end()
        if trial_period_end is None:
            # for example when there is no trial period...
            continue
        now = DateTime()
        if now + days_warning >= trial_period_end:
            # Check if we have already warned about this.
            notified = INotified(current_contract)
            if notified.has_notification(TRIAL_PERIOD_ENDING_NOTIFICATION):
                continue
            notify(TrialPeriodEndingEvent(employee))
            notified.add_notification(TRIAL_PERIOD_ENDING_NOTIFICATION)



__author__ = """Jean-Paul Ladage <j.ladage@zestsoftware.nl>"""
__docformat__ = 'plaintext'

from Acquisition import aq_parent, aq_chain
from Products.Archetypes.atapi import registerType
from zope import component
from zope.interface import implements
from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import DisplayList
from Products.CMFCore.utils import getToolByName

from plonehrm.contracts import config
from plonehrm.contracts.content.contract import Contract
from plonehrm.contracts.content.contract import Contract_schema
from plonehrm.contracts.interfaces import ILetter

Letter_schema = Contract_schema.copy()
Letter_schema['duration'].widget.visible={'view':'visible', 'edit':'invisible'}
Letter_schema['trialPeriod'].widget.visible={'view':'visible', 'edit':'invisible'}


class Letter(Contract):
    """Letter for change of contract
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(Contract,'__implements__',()),)
    implements(ILetter)

    _at_rename_after_creation = True

    schema = Letter_schema

    security.declarePrivate('_templates')
    def _templates(self):
        """Vocabulary for the template field
        """
        tool = getToolByName(self, 'portal_contracts', None)
        if tool is None:
            return []
        else:
            items = [(item.id, item.Title()) for item
                     in tool.listTemplates('letter')]
            return DisplayList(items)


registerType(Letter, config.PROJECTNAME)



__author__ = """Jean-Paul Ladage <j.ladage@zestsoftware.nl>"""
__docformat__ = 'plaintext'

import logging

from zope.i18nmessageid import MessageFactory
from Products.Archetypes import listTypes
from Products.Archetypes.atapi import process_types
from Products.CMFCore import utils as cmfutils
from Products.validation.exceptions import AlreadyRegisteredValidatorError

from plonehrm.contracts import config
from Products.validation import validation
from validators import MaxDaysPerWeek
validation.register(MaxDaysPerWeek('maxDaysPerWeek'))
logger = logging.getLogger('plonehrm.contracts')
logger.debug('Installing Product')
ContractsMessageFactory = MessageFactory(u'contract')

try:
    from plonehrm.contracts import validators
except AlreadyRegisteredValidatorError:
    pass


def initialize(context):
    import content
    from content import tool

    content_types, constructors, ftis = process_types(
        listTypes(config.PROJECTNAME),
        config.PROJECTNAME)

    cmfutils.ContentInit(
        config.PROJECTNAME + ' Content',
        content_types      = content_types,
        permission         = 'plonehrm: Add contract',
        extra_constructors = constructors,
        fti                = ftis,
        ).initialize(context)



from zope.i18n import translate
from zope.interface import implements
from zope.component.interfaces import ObjectEvent

from Products.CMFCore.utils import getToolByName

from plonehrm.contracts import ContractsMessageFactory as _
from plonehrm.contracts.notifications.interfaces import IContractEvent


class NoContractEvent(ObjectEvent):
    implements(IContractEvent)


class ContractEndingEvent(ObjectEvent):
    """Contract is almost ending.
    """
    implements(IContractEvent)

    # This needs to be handled by a (HRM) Manager, e.g. a checklist
    # item specifically for managers needs to be created.
    for_manager = True

    def __init__(self, *args, **kwargs):
        super(ContractEndingEvent, self).__init__(*args, **kwargs)
        create_url = self.object.absolute_url() + \
            '/createObject?type_name=Contract'
        contracts = self.object.restrictedTraverse('@@contracts')
        expires = contracts.expires()
        # Add a message to make a new contract, with the correct link.
        text = _('msg_make_new_contract', u"Make new contract")
        props = getToolByName(self.object, 'portal_properties')
        lang = props.site_properties.getProperty('default_language')
        text = translate(text, target_language=lang)
        self.message = text
        self.link_url = create_url
        self.date = expires


class TrialPeriodEndingEvent(ObjectEvent):
    """Trial period is almost ending.
    """
    implements(IContractEvent)

    # This needs to be handled by a (HRM) Manager, e.g. a checklist
    # item specifically for managers needs to be created.
    for_manager = True

    def __init__(self, *args, **kwargs):
        super(TrialPeriodEndingEvent, self).__init__(*args, **kwargs)
        contracts = self.object.restrictedTraverse('@@contracts')
        trial_period_end = contracts.trial_period_end()
        # Add a warning message.
        toLocalizedTime = self.object.restrictedTraverse(
            '@@plone').toLocalizedTime
        trial_period_end = toLocalizedTime(trial_period_end)
        text = _(u"Trial period ends at ${date}.",
                 mapping=dict(date=trial_period_end))
        props = getToolByName(self.object, 'portal_properties')
        lang = props.site_properties.getProperty('default_language')
        self.message = translate(text, target_language=lang)



import logging

from zope.component import queryMultiAdapter
from Products.CMFCore.utils import getToolByName

logger = logging.getLogger('plonehrm.contracts.events')


def apply_template(object, event, rename=True):
    """After initializing a contract, set text and title based on template.

    If rename is True (the default) we will rename the object after
    setting its title.  Within tests this may fail, so there it can
    help to set it to False.  Yes, this is a hack.
    """
    view = queryMultiAdapter((object, object.REQUEST),
                             name=u'substituter')
    if view is None:
        raise ValueError('Components are not properly configured, could '
                         'not find "substituter" view')

    tool = getToolByName(object, 'portal_contracts', None)
    if tool is None:
        logger.error("portal_contracts cannot be found.")
        return

    # Get the text from the template
    pages = [t for t in tool.listTemplates()
                     if t.getId() == object.template]
    if not pages:
        logger.warn("Template %r cannot be found." % object.template)
        return
    template_page = pages[0]
    template_text = template_page.getText()

    # Save the substituted text on the object.
    resulting_text = view.substitute(template_text)
    object.setText(resulting_text)

    # Set the title to the title of the template (appending 1, 2,
    # 3...), if it has not been set explicitly.
    if object.Title():
        return
    employee = object.get_employee()
    title = template_page.Title()
    if employee is not None:
        numbers = [0]
        for contract in employee.getFolderContents(
            dict(portal_type=object.portal_type)):
            if not contract.Title.startswith(title):
                continue
            number = contract.Title.split(' ')[-1]
            try:
                numbers.append(int(number))
            except ValueError:
                continue
        title += ' %d' % (max(numbers) + 1)
    object.title = title
    # Now we have a title, we rename the object to something nicer
    # than 'contract.2009-05-15.1869364249'.
    if not rename:
        return
    object._renameAfterCreation()



__author__ = """Jean-Paul Ladage <j.ladage@zestsoftware.nl>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Acquisition import aq_inner, aq_parent
from Products.Archetypes.atapi import BaseFolder
from Products.Archetypes.atapi import BaseFolderSchema
from Products.Archetypes.atapi import LinesField
from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import registerType
from Products.CMFCore.utils import ImmutableId
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ModifyPortalContent

from zope.i18n import translate
from zope.interface import implements

from plonehrm.contracts import config
from plonehrm.contracts.interfaces import IContractTool
from plonehrm.contracts import ContractsMessageFactory as _

schema = Schema((
    LinesField(
        name='functions',
        widget=LinesField._properties['widget'](
            label=_(u'contract_label_functions', default=u'Positions'),
        )
    ),
    LinesField(
        name='employmentTypes',
        widget=LinesField._properties['widget'](
            label=_(u'contract_label_employmentTypes',
                    default=u'Types of employment'),
        )
    ),
),
)

ContractTool_schema = BaseFolderSchema.copy() + schema.copy()


class ContractTool(ImmutableId, BaseFolder):
    """Contract tool.

      >>> from plonehrm.contracts.content.tool import ContractTool
      >>> tool = ContractTool()
      >>> tool.Title()
      'Contract templates'

      >>> tool.at_post_edit_script()

    """

    id = 'portal_contracts'
    security = ClassSecurityInfo()
    __implements__ = (BaseFolder.__implements__, )
    implements(IContractTool)

    typeDescription = "ContractTool"
    typeDescMsgId = 'description_edit_contracttool'

    schema = ContractTool_schema

    def __init__(self, *args, **kwargs):
        self.setTitle(translate(_(u'title_portal_contracts',
                                  default=u'Contract templates')))

    def listTemplates(self, type=None):
        """List the templates in this tool.

        Optionally list the templates from a higher level
        portal_contracts tool as well.  This is done by trying to call
        getUseHigherLevelTemplates on the current tool.  This method
        is not available by default, but can be implemented by third
        party products.  Default is False.  If you always want this to
        be True, add a python script getUseHigherLevelTool in the
        portal_skins/custom folder that just returns True.

        We return full objects; returning just ids will fail as we
        then have no way of knowing in which portal_contracts that
        template id is.

        When specifying the parameter 'type' we return only templates
        of that type (presumably 'contract' or 'letter'), otherwise we
        return all.
        """
        templates = self.contentValues()

        try:
            recursive = self.getUseHigherLevelTool()
        except AttributeError:
            recursive = False

        if recursive:
            # Get our grand parent and ask him for a portal_contracts
            # tool.  Note that if we ask our parent, then a
            # getToolByName will return ourselves, which is not what
            # we want.
            grand_parent = aq_parent(aq_parent(aq_inner(self)))
            higher_tool = getToolByName(grand_parent, self.id, None)
            if higher_tool is not None:
                templates += higher_tool.listTemplates()


        # We filter the templates to keep only contract/letter templates.
        filtered = []
        for template in templates:
            try:
                template_type = template.getType()
            except:
                # An old template stays in the contract tool.
                continue
            if type is None or template_type == type:
                filtered.append(template)

        return filtered

    security.declareProtected(ModifyPortalContent, 'indexObject')
    def indexObject(self):
        pass

    security.declareProtected(ModifyPortalContent, 'reindexObject')
    def reindexObject(self, idxs=[]):
        pass

    security.declareProtected(ModifyPortalContent, 'reindexObjectSecurity')
    def reindexObjectSecurity(self, skip_self=False):
        pass


registerType(ContractTool, config.PROJECTNAME)



from plonehrm.notifications.interfaces import IHRMModuleEvent
from plonehrm.notifications.interfaces import IHRMEmailer


class IContractEvent(IHRMModuleEvent):
    pass


class IContractEmailer(IHRMEmailer):
    pass



from Products.CMFCore.utils import getToolByName


def setup(context):
    # This is the main method that gets called by genericsetup.
    if context.readDataFile('plonehrm.contracts.txt') is None:
        return
    site = context.getSite()
    logger = context.getLogger('plonehrm')
    add_currency_property(site, logger)


def add_currency_property(site, logger):
    """Add a currency property to portal_properties.plonehrm_properties.

    We do that in python code instead of in propertiestool.xml to
    avoid overwriting changes by the user.

    Plus: there was a problem importing a string property with
    '&euro;' as value:

    ExpatError: /test/portal_properties: undefined entity: line 7, column 42
    """
    portal_props = getToolByName(site, 'portal_properties')
    props = portal_props.plonehrm_properties
    propname = 'currency'
    if not props.hasProperty(propname):
        value = '&euro;'
        props._setProperty(propname, value, 'string')
        logger.info('Added property %r with default value %r',
                    propname, value)



from Acquisition import Explicit
from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import implements
from zope.viewlet.interfaces import IViewlet
from zope import component

from plonehrm.contracts.content.contract import ContractHourSpread

def compare_dates(a, b):
    return cmp(a.getStartdate(), b.getStartdate())


def sort_contracts(contracts):
    """Sort Contracts and Letters.

    This is a helper method that is easier to test (and write).

    Wanted:

    - future contract
    - current contract
      - second change to current contract
      - first change to current contract
    - previous contract
      - only change to previous contract

    Well, the indentation will be done with css or something, but the
    above order should be there.

    Make mock classes:

    >>> class MockContract(object):
    ...     portal_type = 'Contract'
    ...     def __init__(self, date):
    ...         self.date = date
    ...     def getStartdate(self):
    ...         return self.date
    ...     def __repr__(self):
    ...         return '%s from %r' % (self.portal_type, str(self.date))
    >>> class MockLetter(MockContract):
    ...     portal_type = 'Letter'

    Make sample objects.  Most in the past and one in the future:

    >>> contract_0 = MockContract(DateTime(1999, 1, 1))
    >>> letter_0 = MockLetter(DateTime(1999, 5, 16))
    >>> contract_1 = MockContract(DateTime(2001, 1, 1))
    >>> letter_1 = MockLetter(DateTime(2001, 5, 16))
    >>> contract_2 = MockContract(DateTime(2001, 7, 1))
    >>> letter_2a = MockLetter(DateTime(2001, 8, 16))
    >>> letter_2b = MockLetter(DateTime(2001, 10, 16))
    >>> contract_3 = MockContract(DateTime(2100, 1, 1))

    Now some easy tests:

    >>> sort_contracts([])
    []
    >>> sort_contracts([contract_1])
    [Contract from '2001/01/01']
    >>> sort_contracts([letter_1])
    [Letter from '2001/05/16']

    Contracts should be listed newest first.

    >>> sort_contracts([contract_3, contract_1, contract_2]) == [contract_3, contract_2, contract_1]
    True

    Same for letters:

    >>> sort_contracts([letter_2a, letter_1, letter_2b]) == [letter_2b, letter_2a, letter_1]
    True

    A change letter should be listed *after* its contract.

    >>> sort_contracts([letter_1, contract_1])
    [Contract from '2001/01/01', Letter from '2001/05/16']
    >>> sort_contracts([letter_2a, contract_2, letter_2b])
    [Contract from '2001/07/01', Letter from '2001/10/16', Letter from '2001/08/16']

    Now the middle ones.

    >>> contracts = [contract_1, contract_2, letter_1, letter_2a, letter_2b]
    >>> sort_contracts(contracts)
    [Contract from '2001/07/01', Letter from '2001/10/16', Letter from '2001/08/16', Contract from '2001/01/01', Letter from '2001/05/16']

    Add the last one.

    >>> contracts.append(contract_3)
    >>> sort_contracts(contracts)
    [Contract from '2100/01/01', Contract from '2001/07/01', Letter from '2001/10/16', Letter from '2001/08/16', Contract from '2001/01/01', Letter from '2001/05/16']

    See if adding Contract zero is for some reason difficult.

    >>> sort_contracts(contracts + [contract_0])
    [Contract from '2100/01/01', Contract from '2001/07/01', Letter from '2001/10/16', Letter from '2001/08/16', Contract from '2001/01/01', Letter from '2001/05/16', Contract from '1999/01/01']

    Check if starting with a Letter adds difficulties (thought doing
    so would be strange.

    >>> sort_contracts(contracts + [letter_0])
    [Contract from '2100/01/01', Contract from '2001/07/01', Letter from '2001/10/16', Letter from '2001/08/16', Contract from '2001/01/01', Letter from '2001/05/16', Letter from '1999/05/16']

    """
    contracts.sort(cmp=compare_dates, reverse=True)
    sorted = []
    changes = []
    for con in contracts:
        if con.portal_type == 'Contract':
            # First add the contract,
            sorted.append(con)
            # then add the temporary list of change letters,
            sorted += changes
            # then empty that listit
            changes = []
        else:
            changes.append(con)
    # For safety (and testing)
    sorted += changes
    return sorted


class ContractView(BrowserView):

    def first_contract(self):
        today = DateTime()
        filter = dict(portal_type='Contract')
        contracts = self.context.contentValues(filter=filter)
        if len(contracts) == 0:
            return None
        contracts.sort(cmp=compare_dates)
        return contracts[0]
        
    def start_employment(self):
        contract = self.first_contract()
        if contract is not None:
            start = contract.getStartdate()
            return start
        return None
        
    def sum_worktime(self):
        
        start_date = self.context.workStartDate
        if start_date is not None:
           worktime = start_date
        else:
           worktime = self.start_employment()
    
        if worktime is not None:
            today = DateTime()            
            expire = self.expires()          
            if expire is not None and expire < today:
                date_to_compare = expire
            else: 
                date_to_compare = today
            
            worktime_year = date_to_compare.year() - worktime.year() 
            worktime_month = date_to_compare.month() - worktime.month()
            
            if worktime_month < 0:
                worktime_month = worktime_month+12
                worktime_year = worktime_year-1
          
            return {'year': worktime_year,
                    'month': worktime_month}
            
        return None
        

    def current_contract(self, includeChangeLetters=True):
        """Return a link to the contract"""
        today = DateTime()
        # Get a list of contracts with start dates before today.
        if includeChangeLetters:
            filter=dict(portal_type=('Contract', 'Letter'))
        else:
            filter=dict(portal_type='Contract')
        contracts = self.context.contentValues(filter=filter)
        candidates = [(c.getStartdate(), c) for c in contracts
                      if today > c.getStartdate() > 0]
        if len(candidates) > 0:
            # sort by date
            candidates.sort()
            dummy, contract = candidates[-1]
            return contract
        return None

    def expires(self):
        """Return the expiration date of the current contract.

        ChangeLetters can have no duration as they cannot change the
        duration of the Contract they are changing.  So ignore those.

        XXX If the current contract almost expires and there already
        is a new contract, then we should probably take that date as
        expiry date.
        """
        contract = self.current_contract(includeChangeLetters=False)
        if contract is not None:
            return contract.expiry_date()
        return None

    def default_start_date(self):
        """ Takes the current contract expiry data and adds one day.
        """
        last_contract_expire = self.expires()
        if not last_contract_expire:
            return

        return last_contract_expire + 1
        
    def trial_period_end(self):
        """Return the expiration date of the current contract.

        ChangeLetters can have no duration as they cannot change the
        duration of the Contract they are changing.  So ignore those.

        XXX If the current contract almost expires and there already
        is a new contract, then we should probably take that date as
        expiry date.
        """
        contract = self.current_contract(includeChangeLetters=False)
        if contract is not None:
            start = contract.getStartdate()
            trialPeriod = contract.getTrialPeriod()
            if start is None or trialPeriod == 0:
                return None
            year, month, day = start.year(), start.month(), start.day()
            year = year + (month + trialPeriod - 1) / 12
            month = 1 + (month + trialPeriod - 1) % 12
            # Try not to return e.g. 31 February ...
            first_of_month =  DateTime(year, month, 1)
            return first_of_month + day - 1
        return None


class ContractViewlet(Explicit, ContractView):
    implements(IViewlet)
    render = ViewPageTemplateFile('contract.pt')

    def __init__(self, context, request, view=None, manager=None):
        self.__parent__ = view
        self.context = context
        self.request = request
        self.view = view
        self.manager = manager

        # Attribute used to know which kind of view is displayed.
        # It can have four values: list, add_contract
        # add_letter, settings and endEmployment.
        self.view_mode = "list"

    def update(self):
        pass

    def number(self):
        """Return the number of contracts.
        """
        cat = getToolByName(self.context, 'portal_catalog')
        path = '/'.join(self.context.getPhysicalPath())
        brains = cat.searchResults({'portal_type': 'Contract', 'path': path})
        if brains:
            return len(brains)

    def wage(self):
        """ Return the wage
        """
        if self.current_contract():
            return self.current_contract().getWage()

    def get_function(self):
        """ Return function
        """

        if self.current_contract():
            return self.current_contract().getFunction()

    def add_url(self):
        """ Add new contract
        """
        # check Add permission on employee
        # return None if we don't have permission
        mtool = getToolByName(self.context, 'portal_membership')
        if mtool.checkPermission('plonehrm: Add contract', self.context):
            url = self.context.absolute_url() + '/createObject?type_name=Contract'
            return url

    def contract_list(self):
        return self.context.contentValues(
            filter=dict(portal_type=('Contract', 'Letter')))

    def sorted_contracts(self):
        """Sort contracts and letters
        """
        candidates = sort_contracts(self.contract_list())
        results = []
        if len(candidates) > 0:
            for contract in candidates:
                con = dict(contract=contract,
                           typ=contract.portal_type.lower())
                results.append(con)
        return results

    def start_date(self):
        """ Returns the date when the employee started working.
        """
        return self.context.getWorkStartDate()

    def can_create_contract(self):
        """ This function checks if templates, functions and employment
        types have already been set. It does not check security.
        """
        return bool(self.get_templates()) and \
               bool(self.get_functions()) and \
               bool(self.get_employment_types())

    def get_templates(self, type='contract'):
        """ Returns the list of available templates.
        """
        tool = getToolByName(self, 'portal_contracts', None)
        if tool is None:
            return []
        else:
            return tool.listTemplates(type)

    def get_functions(self):
        """Vocabulary for the functions field
        """
        tool = getToolByName(self, 'portal_contracts', None)
        if tool is None:
            return []
        else:
            return tool.getFunctions()

    def get_employment_types(self):
        """Vocabulary for the employmentType field
        """
        tool = getToolByName(self, 'portal_contracts', None)
        if tool is None:
            return []
        else:
            return tool.getEmploymentTypes()

    def base_contract(self):
        """Get the current contract of the parent Employee.
        """
        view = component.getMultiAdapter((self.context, self.request),
                                         name=u'contracts')
        return view.current_contract()

    def default_wage(self):
        """Get the wage of the parent Employee.
        """
        base = self.base_contract()
        if base is None or base == self:
            return '0.00'
        return base.getWage()

    def default_function(self):
        """Get the function of the parent Employee.
        """
        base = self.base_contract()
        if base is None or base == self:
            return ''
        return base.getFunction()

    def default_hours(self):
        """Get the hours of the parent Employee.
        """
        base = self.base_contract()
        if base is None or base == self:
            return ''
        return base.getHours()

    def default_employment_type(self):
        """Get the employment type of the parent Employee.
        """
        base = self.base_contract()
        if base is None or base == self:
            return ''
        return base.getEmploymentType()

    def default_days_per_week(self):
        """Get the days per week of the parent Employee.
        """
        base = self.base_contract()
        if base is None or base == self:
            return 0
        return base.getDaysPerWeek()

    def default_hour_spread(self):
        """ Returns the ContractHourSpread object linked to the contract.
        """
        base = self.base_contract()
        if base is None or base == self:
            return ContractHourSpread()
        return base.hour_spread

    def is_arbo_manager(self):
        """ Checks if the user has arbo manager rights.
        """
        membership = getToolByName(self.context, 'portal_membership')
        return membership.checkPermission('plonehrm: manage Arbo content',
                                          self.context)



from Products.validation.interfaces import ivalidator
from Products.validation import validation
from Products.validation.validators.RegexValidator import RegexValidator
from Products.plonehrm import PloneHrmMessageFactory as _
from zope.i18n import translate

class MaxDaysPerWeek:
    __implements__ = (ivalidator,)
    
    def __init__(self, name):
        self.name = name
        
    def __call__(self, value, *args, **kwargs):

       value = int(value)
       maxdays = 7
       if value < 0 or value > maxdays:
          error = _("Please enter 0 to 7 days.")
          return translate(error)
       return value

# Allow both commas and dots as decimal points.  And do not allow
# exponentials (like 5.0e2).

isCurrency = RegexValidator(
    'isCurrency',
    r'^([+-]?)(?=\d|[,\.]\d)\d*([,\.]\d*)?$',
    title='', description='',
    errmsg='is not a decimal number.')

validation.register(isCurrency)



import unittest
from zope.testing import doctest
from zope.component import testing

def test_suite():
    return unittest.TestSuite((
        doctest.DocTestSuite(module='plonehrm.contracts.browser.contract'),
        doctest.DocTestSuite(module='plonehrm.contracts.content.contract'),
        doctest.DocTestSuite(module='plonehrm.contracts.content.letter'),
        doctest.DocTestSuite(module='plonehrm.contracts.notifications.checkers'),
        doctest.DocTestSuite(module='plonehrm.contracts.notifications.events'),
        ))



from Acquisition import aq_inner
from DateTime import DateTime
from kss.core import kssaction, KSSView
from zope.event import notify
from Products.Archetypes.event import ObjectInitializedEvent
from Products.CMFCore.utils import getToolByName
from zope.i18n import translate

from plonehrm.contracts import ContractsMessageFactory as _

class ContractKssView(KSSView):
    """ Class managing KSS actions for the contract viewlet.
    """

    def replace_viewlet(self, mode = 'list'):
        """ This method refreshes the content of the
        contract viewlet.
        The 'mode' parameter is used to know which type of view is
        used.
        """

        # We check that the mode used is a correct one.
        if not mode in ['list', 'add_contract', 'add_letter',
                        'settings', 'endEmployment']:
            mode = 'list'

        # First, we get the viewlet manager.
        view = self.context.restrictedTraverse('@@plonehrm.contracts')

        # We configure it.
        view.view_mode = mode

        # We get the content displayed by in the viewlet.
        rendered = view.render()

        # We replace the content of the viewlet by the new one.
        core = self.getCommandSet('core')
        selector = core.getHtmlIdSelector('plonehrmContractViewlet')
        core.replaceHTML(selector, rendered)

    @property
    def lang(self):
        props = getToolByName(self.context, 'portal_properties')
        return props.site_properties.getProperty('default_language')     

    @kssaction
    def show_list(self, hide_status=True):
        """ Shows the contract list in the viewlet.
        """
        
        #Hides the previous status message.
        if hide_status:
            self.getCommandSet('plone').issuePortalMessage('')

        self.replace_viewlet('list')

    @kssaction
    def show_add_letter(self):
        """ Shows the add letter form in the viewlet.
        """
        self.replace_viewlet('add_letter')

    @kssaction
    def show_add_contract(self):
        """ Shows the add letter form in the viewlet.
        """
        self.replace_viewlet('add_contract')

    @kssaction
    def show_settings(self):
        """ Shows the settings form in the viewlet.
        """
        self.replace_viewlet('settings')

    @kssaction
    def show_end_employment(self):
        """ Shows the form to end employment.
        """
        self.replace_viewlet('endEmployment')

    @kssaction
    def save_settings(self):
        """ Saves the settings for the employee.
        The form is not passed as a parameter, we use
        kssSubmitForm to send the data.
        """
        form = self.request.form
        core = self.getCommandSet('core')

        field_base = 'contract_settings_start_date'
        if form.get(field_base + '_year', '0000') != '0000':
            year = int(self.request.get(field_base + '_year'))
            month = int(self.request.get(field_base + '_month'))
            day = int(self.request.get(field_base + '_day'))
            try:
                date = DateTime(year, month, day)
            except:
                selector = core.getHtmlIdSelector(
                    'contract_viewlet_settings_baddate_error')
                core.setAttribute(selector, 'class', 'errormessage')

                # Show error status message
                message = translate(_(u'msg_error_saving_settings',
                                      u'Errors have been found when saving settings'),
                                    target_language=self.lang)
                self.getCommandSet('plone').issuePortalMessage(message,
                                                               msgtype='Error')
                
                return

            if date and not date.isPast():
                # Displays an error message
                selector = core.getHtmlIdSelector(
                    'contract_viewlet_settings_pastdate_error')
                core.setAttribute(selector, 'class', 'errormessage')

                message = translate(_(u'msg_error_saving_settings',
                                      u'Errors have been found when saving settings'),
                                    target_language=self.lang)
                self.getCommandSet('plone').issuePortalMessage(message,
                                                               msgtype='Error')
                return
        else:
            date = None

        # Save the date and display the contract list again.
        self.context.setWorkStartDate(date)

        if date is None:
            message = ''
        else:
            message = translate(_(u'msg_settings_savec',
                                  u'Settings have been saved'),
                                target_language=self.lang)

        self.getCommandSet('plone').issuePortalMessage(message)
        self.show_list(hide_status=False)

    @kssaction
    def end_employment(self):
        """ Sets date and reason for employment end.
        """
        form = self.request.form
        core = self.getCommandSet('core')

        employee = aq_inner(self.context)
        fields = ['end_employment_reason',
                  'contract_end_employment_date_day',
                  'contract_end_employment_date_month',
                  'contract_end_employment_date_year']

        for field in fields:
            if not field in form:
                # Should not happen.
                return

        # Hide previous errors.
        errors = ['contract_viewlet_end_date_invalid_error',
                  'contract_viewlet_end_date_empty_error']
        
        for error in errors:
            selector = core.getHtmlIdSelector(error)
            core.setAttribute(selector, 'class', 'dont-show')

        # Potential error message.
        message = translate(_(u'msg_employee_errors',
                              u'Errors have been found while terminating employment.'),
                            target_language=self.lang)

        # Check that a date has been set.
        if form['contract_end_employment_date_day'] == '00':
            selector = core.getHtmlIdSelector(
                'contract_viewlet_end_date_empty_error')
            core.setAttribute(selector, 'class', 'errormessage')
            self.getCommandSet('plone').issuePortalMessage(message,
                                                           msgtype='Error')
            return

        # Check that the date is correct.
        try:
            date = DateTime(int(form['contract_end_employment_date_year']),
                            int(form['contract_end_employment_date_month']),
                            int(form['contract_end_employment_date_day']))
        except:
            selector = core.getHtmlIdSelector(
                'contract_viewlet_end_date_invalid_error')
            core.setAttribute(selector, 'class', 'errormessage')
            self.getCommandSet('plone').issuePortalMessage(message,
                                                           msgtype='Error')
            return

        # Set end date/reason
        employee.setEndEmploymentDate(date)
        employee.setEndEmploymentReason(form['end_employment_reason'])

        # Change employee workflow.
        try:
            workflowTool = getToolByName(employee, "portal_workflow")
            workflowTool.doActionFor(employee, "deactivate")
        except:
            # The employment was already terminated.
            pass

        # Show the success message.
        message = translate(_(u'msg_employee_ended',
                              u'Employment has been terminated.'),
                            target_language=self.lang)

        self.getCommandSet('plone').issuePortalMessage(message)

        # Refresh the viewlet.
        self.show_list(hide_status=False)
        
        # Refresh the checklist viewlet.
        view = self.context.restrictedTraverse('@@plonehrm.checklist')

        # We configure it.
        view.view_mode = 'list'
        view.editedItem = None

        # We get the content displayed by in the viewlet.
        rendered = view.render()

        # We replace the content of the viewlet by the new one.
        core = self.getCommandSet('core')
        selector = core.getHtmlIdSelector('checklist')
        core.replaceHTML(selector, rendered)




    def _validate_field(self, value, errormsg, type='int', errors = []):
        """ This method tries to cast value into the asked type.
        If casting is not possible, the error message is appended into
        the errors list.
        """
        if value:
            try:
                if type == 'int':
                    int(value)
                elif type == 'float':
                    tmp = '.'.join(value.split(','))
                    float(tmp)
            except ValueError:
                errors.append(errormsg)

    @kssaction
    def add_contract(self):
        """ Method called when a new contract is added.
        """
        self.add_contract_or_letter()

    @kssaction
    def add_letter(self):
        """ Method called when a new letter is added.
        """
        self.add_contract_or_letter(False)

    def add_contract_or_letter(self, is_contract = True):
        """ Action called when a contract/letter is saved.
        """
        form = self.request.form
        core = self.getCommandSet('core')

        field_base = 'contract_form_'

        title = form.get(field_base + 'title')
        template = form.get(field_base + 'template')
        wage = form.get(field_base + 'wage')
        year = form.get(field_base + 'startdate_year')
        month = form.get(field_base + 'startdate_month')
        day = form.get(field_base + 'startdate_day')
        function = form.get(field_base + 'function')
        duration = form.get(field_base + 'duration')
        trial_period = form.get(field_base + 'trial_period')
        employment_type = form.get(field_base + 'employment_type')
        number_hours = form.get(field_base + 'number_hours')
        workdays = form.get(field_base + 'workdays')

        # Before displaying any error, we clean the previous ones.
        for errorMsg in ['contract_viewlet_no_template',
                         'contract_viewlet_wage_no_float',
                         'contract_viewlet_invalid_date',
                         'contract_viewlet_duration_no_integer',
                         'contract_viewlet_hours_no_integer',
                         'contract_viewlet_workdays_no_integer',
                         'contract_viewlet_incorrect_workdays',
                         'contract_viewlet_spread_to_high',
                         'contract_viewlet_more_than_24']:
            selector = core.getHtmlIdSelector(errorMsg)
            core.setAttribute(selector, 'class', 'dont-show')

        errors = []

        if template == '':
            errors.append('contract_viewlet_no_template')

        self._validate_field(wage, 'contract_viewlet_wage_no_float',
                             'float', errors)

        if form.get(field_base + 'startdate_year', '0000') != '0000':
            try:
                date = DateTime(int(year), int(month), int(day))
            except:
                errors.append('contract_viewlet_invalid_date')
                date = None
        else:
            date = None

        if is_contract:
            # Those two fields are not present in
            # the letter form.
            self._validate_field(
                duration, 'contract_viewlet_duration_no_integer',
                'int', errors)

            # We do not raise any error as this case should not append,
            # the only values possible for this field are 0, 1 or 2.
            self._validate_field(trial_period, '', 'int', errors)


        self._validate_field(number_hours, 'contract_viewlet_hours_no_integer',
                         'int', errors)

        try:
            tmp = int(workdays)
            if tmp < 1 or tmp > 7:
                errors.append('contract_viewlet_incorrect_workdays')
        except:
            if workdays:
                errors.append('contract_viewlet_workdays_no_integer')

        # We check that number of hours specified is correct.
        if 'contract_form_working_schedule' in form:
            total = 0
            if form['contract_form_working_schedule'] == 'manual':
                rows = ['odd']
            elif form['contract_form_working_schedule'] == 'manual_oddeven':
                rows = ['odd', 'even']
            else:
                rows = []

            for row in rows:
                for day in range(0, 7):
                    try:
                        field = float(form['schedule_' + row + '_' + str(day)])
                    except:
                        field = 0
                    if field in form:                           
                        total += field
                        if field > 24:
                            errors.append('contract_viewlet_more_than_24')

            if rows:
                total = total / len(rows)
                if total > int(number_hours):
                    errors.append('contract_viewlet_spread_to_high')

        # We display the error messages.
        for errorMsg in errors:
            selector = core.getHtmlIdSelector(errorMsg)
            core.setAttribute(selector, 'class', 'errormessage')

        if len(errors) > 0:
            if is_contract:
                message = translate(_(u'msg_error_contract',
                                      u'Errors were found while creating the contract'),
                                target_language=self.lang)
            else:
                message = translate(_(u'msg_error_letter',
                                      u'Errors were found while creating the letter'),
                                target_language=self.lang)
                
            self.getCommandSet('plone').issuePortalMessage(message,
                                                           msgtype='Error')
            
            return

        # If no error has been found, we create the contract and display
        # the contract list.
        if is_contract:
            new_id = self.context.generateUniqueId('Contract')
            self.context.invokeFactory("Contract", id=new_id,
                                       title='contract_tmp')
        else:
            new_id = self.context.generateUniqueId('Letter')
            self.context.invokeFactory("Letter", id=new_id,
                                       title='letter_tmp')

        contract = getattr(self.context, new_id)

        contract.setTitle(title)
        contract.setTemplate(template)
        contract.setWage(wage)
        contract.setFunction(function)
        contract.setEmploymentType(employment_type)
        contract.setHours(number_hours)
        contract.setDaysPerWeek(workdays)
        contract.setStartdate(date)

        if is_contract:
            contract.setDuration(duration)
            contract.setTrialPeriod(trial_period)

        contract.unmarkCreationFlag()
        contract._renameAfterCreation()
        notify(ObjectInitializedEvent(contract))

        # Adds informations for hours spread.
        if 'contract_form_working_schedule' in form:
            contract.hour_spread.update_from_form(form)

        # Shows status message
        if is_contract:
            message = translate(_(u'msg_contract_added',
                                  u'Contract added'),
                                target_language=self.lang)
        else:
            message = translate(_(u'msg_letter_added',
                                  u'Letter added'),
                                target_language=self.lang)
        self.getCommandSet('plone').issuePortalMessage(message)
     

        self.show_list(hide_status=False)

    @kssaction
    def change_schedule_table(self, value):
        """ Show or hide the schedule table.
        """

        if value == 'auto':
            css = 'dont-show'
        elif value in ['manual', 'manual_oddeven']:
            css = 'listing'
        else:
            return
        
        core = self.getCommandSet('core')
        selector = core.getHtmlIdSelector('contract_viewlet_manual_schedule')
        core.setAttribute(selector, 'class', css)

        if not value == 'auto':
            if value == 'manual':
                css = 'dont-show'
                first_row = _(u'label_normal', u'normal')
            else:
                css = ''
                first_row = _(u'label_odd', u'odd')
            
            # We hide the second row which is useless.
            selector = core.getHtmlIdSelector(
                'contract_viewlet_manual_schedule_even')
            core.setAttribute(selector, 'class', css)

            selector = core.getHtmlIdSelector(
                'contract_viewlet_manual_schedule_odd_first_row')
            core.replaceInnerHTML(selector,
                                  translate(first_row,
                                            target_language=self.lang))



try:
    import zope.annotation
except ImportError:
    # BBB for Zope 2.9
    import zope.app.annotation
    import zope.app.annotation.interfaces
    import sys
    sys.modules['zope.annotation'] = zope.app.annotation
    sys.modules['zope.annotation.interfaces'] = zope.app.annotation.interfaces



from zope.interface import Interface


class IContractView(Interface):
    """Provide a view for an Employee specifically for Contracts.
    """

    def current_contract():
        """Return a link to the contract"""

    def expires():
        """Return the expiration date of the current contract."""

    def trial_period_end():
        """Return the end of the trial period of the current contract."""


class IContractViewlet(Interface):

    def current_contract():
        """Return a link to the contract"""

    def expires():
        """Return the expiration date of the current contract."""

    def number():
        """Return the number of contracts.
        """

    def wage():
        """ Return the wage
        """

    def get_function():
        """ Return function
        """

    def add_url():
        """ Add new contract
        """

    def contract_list():
        """Return a list of contracts"""

    def sorted_contracts():
        """Return a sorted list of contracts and letters"""



from setuptools import setup, find_packages
import os.path


def get_file_contents_from_main_dir(filename):
    file_path = os.path.join('plonehrm', 'contracts', filename)
    this_file = open(file_path)
    contents = this_file.read().strip()
    this_file.close()
    return contents

version = get_file_contents_from_main_dir('version.txt')
history = get_file_contents_from_main_dir('HISTORY.txt')
readme = get_file_contents_from_main_dir('README.txt')
long = "%s\n\n\n%s" % (readme, history)


setup(name='plonehrm.contracts',
      version=version,
      description="Contracts for Plone HRM",
      long_description=long,
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Zest Software',
      author_email='info@zestsoftware.nl',
      url='http://zestsoftware.nl',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plonehrm'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'collective.autopermission',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



__author__ = """Jean-Paul Ladage <j.ladage@zestsoftware.nl>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Acquisition import aq_chain, aq_inner
from DateTime import DateTime
from Products.Archetypes.utils import IntDisplayList
from Products.Archetypes.interfaces import IBaseContent
from Products.Archetypes.atapi import BaseContent
from Products.Archetypes.atapi import BaseSchema
from Products.Archetypes.atapi import CalendarWidget
from Products.Archetypes.atapi import DateTimeField
from Products.Archetypes.atapi import DecimalWidget
from Products.Archetypes.atapi import IntegerWidget
from Products.Archetypes.atapi import DisplayList
from Products.Archetypes.atapi import FixedPointField
from Products.Archetypes.atapi import IntegerField
from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import SelectionWidget
from Products.Archetypes.atapi import StringField
from Products.Archetypes.atapi import TextField
from Products.Archetypes.atapi import RichWidget
from Products.Archetypes.atapi import registerType
from Products.CMFCore.utils import getToolByName
from Products.plonehrm.interfaces import IEmployee
from zope import component
from zope.interface import implements
from zope.annotation.interfaces import IAnnotations
from persistent.list import PersistentList
from persistent.dict import PersistentDict
from persistent import Persistent

from plonehrm.contracts import config
from plonehrm.contracts.interfaces import IContract
from plonehrm.contracts import ContractsMessageFactory as _

schema = Schema((
    StringField(name='template',
        required=1,
        vocabulary='_templates',
        read_permission='plonehrm: View hrm content',
        write_permission='plonehrm: Modify contract',
        widget=SelectionWidget(
            format='select',
            condition='not:object/template_chosen',
            label=_(u'contract_label_template', default=u'Template'),
        ),
    ),
    FixedPointField(
        name='wage',
        read_permission='plonehrm: View hrm content',
        write_permission='plonehrm: Modify contract',
        validators = ('isCurrency', ),
        default_method='default_wage',
        widget=DecimalWidget(
            condition='not:object/template_chosen',
            label=_(u'contract_label_wage', default=u'Wage'),
        )
    ),
    StringField(
        name='function',
        read_permission='plonehrm: View hrm content',
        write_permission='plonehrm: Modify contract',
        default_method='default_function',
        widget=SelectionWidget(
            format='select',
            condition='not:object/template_chosen',
            label=_(u'contract_label_function', default=u'Function'),
        ),
        vocabulary='_available_functions'
    ),
    DateTimeField(
        name='startdate',
        read_permission='plonehrm: View hrm content',
        write_permission='plonehrm: Modify contract',
        widget=CalendarWidget(
            condition='not:object/template_chosen',
            show_hm=0,
            label=_(u'contract_label_startdate', default=u'Start date'),
        )
    ),
    IntegerField(
        name='duration',
        read_permission='plonehrm: View hrm content',
        write_permission='plonehrm: Modify contract',
        widget=IntegerWidget(
            condition='not:object/template_chosen',
            label=_(u'contract_label_duration', default=u'Duration (months)'),
        )
    ),
    IntegerField(
        name='trialPeriod',
        read_permission='plonehrm: View hrm content',
        write_permission='plonehrm: Modify contract',
        default=0,
        widget=SelectionWidget(
            format='select',
            condition='not:object/template_chosen',
            label=_(u'contract_label_trial_period',
                    default=u'Trial period (months)'),
        ),
        vocabulary='_trial_period_vocabulary',
    ),
    StringField(
        name='employmentType',
        read_permission='plonehrm: View hrm content',
        write_permission='plonehrm: Modify contract',
        default_method='default_employment_type',
        widget=SelectionWidget(
            condition='not:object/template_chosen',
            format='select',
            label=_(u'contract_label_employmentType',
                    default=u'Type of employment'),

        ),
        vocabulary='_available_employment_types'
    ),
    IntegerField(
        name='hours',
        read_permission='plonehrm: View hrm content',
        write_permission='plonehrm: Modify contract',
        default_method='default_hours',
        widget=IntegerWidget(
            condition='not:object/template_chosen',
            label=_(u'contract_label_hours', default=u'Number of hours'),
        )
    ),
    IntegerField(
        name='daysPerWeek',
        read_permission='plonehrm: View hrm content',
        write_permission='plonehrm: Modify contract',
        default_method='default_days_per_week',
        validators = ('maxDaysPerWeek', ),
        widget=IntegerWidget(
            condition='not:object/template_chosen',
            label=_(u'contract_label_days_per_week',
                    default=u'Number of workdays per week'),
        )
    ),
    TextField(name='text',
        required=False,
        seachable=False,
        primary=True,
        default_output_type = 'text/x-html-safe',
        widget = RichWidget(
                description = '',
                label = _(u'label_body_text', default=u'Body Text'),
                rows = 25,
                visible = {'view': 'visible', 'edit': 'invisible'},
                condition='object/template_chosen',
        ),
    )
),
)

Contract_schema = BaseSchema.copy() + schema.copy()
# We take the title of the chosen template usually, but this can be
# overridden by specifically filling in a title.  But at least the
# title is now not required:
Contract_schema['title'].required = False


class ContractHourSpread(Persistent):
    def __init__(self):
        # Mode can be manual, manual_oddeven or auto.
        self.mode = 'auto'

        # Number of hours worked by week.
        self.total_hours = 0

        # Stores the number of hours per day.
        # Each cell is referenced as week_type + '_' + day number.
        # For example, 'even_0' defines number of hours worked on
        # monday of even weeks. 'odd_3' defines number of jours
        # worked on thurday for odd weeks.
        self.hours = {}
        for week in ['odd', 'even']:
            for day in range(0, 7):
                self.hours[week + '_' + str(day)] = 0

    def update_from_form(self, form):
        """ Takes the form submitted with the contract viewlet and
        updates data.

        This method does not raise any error has the form should have been
        checked previously by the KSS action called when submitting
        the form.
        If problems are found, this method dies silently.
        """

        def quarter_round(f):
            """ Takes a float and returns it rounded to the quarter
                x.0, x.25, x.50, x.75
            """
            int_part = int(f)
            float_part = f - int_part
            
            for i in range(0, 5):
                if float_part > (0.25 * (i - 1)) and float_part < (0.25 * i):
                    float_part = i and 0.25 * (i - 1) or 0.0
                    break

            return int_part + float_part


        # First, we check that all necessary fields are here.
        fields = ['contract_form_workdays',
                  'contract_form_number_hours',
                  'contract_form_working_schedule']

        for week in ['odd', 'even']:
            for day in range(0, 7):
                fields. append('schedule_' + week + '_' + str(day))

        for field in fields:
            if not field in form:
                # Should not happen.
                return
            if not form[field]:
                form[field] = 0

        self.mode = form['contract_form_working_schedule']
        try:
            hours = float(form['contract_form_number_hours'])
            workdays = int(form['contract_form_workdays'])
        except:
            # Should not happen.
            return

        if self.mode == 'auto':
            for week in ['odd', 'even']:
                for day in range(0, workdays):
                    self.hours[week + '_' + str(day)] = \
                                    quarter_round(hours / workdays)
        elif self.mode == 'manual':
            for day in range(0, 7):
                self.hours['odd_' + str(day)] = float(form['schedule_odd_' + \
                                                           str(day)])
                self.hours['even_' + str(day)] = float(form['schedule_odd_' + \
                                                            str(day)])
        elif self.mode == 'manual_oddeven':
            for day in range(0, 7):
                self.hours['odd_' + str(day)] = float(form['schedule_odd_' + \
                                                           str(day)])
                self.hours['even_' + str(day)] = float(form['schedule_even_' + \
                                                            str(day)])

    def get_value_from_field(self, field):
        key = field.split('schedule_')[-1]
        if key in self.hours:
            return self.hours[key]

class Contract(BaseContent):
    """
    """
    security = ClassSecurityInfo()
    __implements__ = (BaseContent.__implements__, )
    implements(IBaseContent, IContract)
    _at_rename_after_creation = True

    schema = Contract_schema

    def getWage(self):
        """Override the default getting to return a comma in some cases.

        We just look at the default language and check if it is in a
        list of languages that we know use commas in their currencies.
        """
        wage = self.getField('wage').get(self)
        language_tool = getToolByName(self, 'portal_languages')
        if language_tool:
            language = language_tool.getDefaultLanguage()
        else:
            language = 'en'
        if language in ('nl', 'de', 'fr'):
            wage = wage.replace('.', ',')
        return wage

    security.declarePrivate('_templates')
    def _templates(self):
        """Vocabulary for the template field
        """
        tool = getToolByName(self, 'portal_contracts', None)
        if tool is None:
            return []
        else:
            items = [(item.id, item.Title()) for item
                     in tool.listTemplates('contract')]
            return DisplayList(items)

    security.declarePublic('template_chosen')
    def template_chosen(self):
        """Determine if the template (a string) has been chosen yet.
        """
        return len(self.getTemplate()) > 0

    security.declarePrivate('_available_functions')
    def _available_functions(self):
        """Vocabulary for the functions field
        """
        tool = getToolByName(self, 'portal_contracts', None)
        if tool is None:
            return []
        else:
            return tool.getFunctions()

    security.declarePrivate('_available_employment_types')
    def _available_employment_types(self):
        """Vocabulary for the employmentType field
        """
        tool = getToolByName(self, 'portal_contracts', None)
        if tool is None:
            return []
        else:
            return tool.getEmploymentTypes()

    security.declarePrivate('_trial_period_vocabulary')
    def _trial_period_vocabulary(self):
        """Vocabulary for the trialPeriod field
        """
        return IntDisplayList([(0, '0'), (1, '1'), (2, '2')])

    def get_employee(self):
        """Get the employee that this contract is in.

        Note that we probably are in the portal_factory, so our direct
        parent is not an Employee.  But we can traverse up the
        acquisition chain to find it.
        """
        for parent in aq_chain(aq_inner(self)):
            if IEmployee.providedBy(parent):
                return parent

    def base_contract(self):
        """Get the current contract of the parent Employee.
        """
        parent = self.get_employee()
        if parent is None:
            return
        # XXX: Remove view lookup
        # A content class should not need to be aware of the request in
        # order to preserve separation of concerns.  The subsitution
        # mechanism here should be replaced with a component that doesn't
        # know or care about the request - Rocky
        view = component.getMultiAdapter((parent, self.REQUEST),
                                         name=u'contracts')
        return view.current_contract()

    def default_wage(self):
        """Get the wage of the parent Employee.
        """
        base = self.base_contract()
        if base is None or base == self:
            return '0.00'
        return base.getWage()

    def default_function(self):
        """Get the function of the parent Employee.
        """
        base = self.base_contract()
        if base is None or base == self:
            return ''
        return base.getFunction()

    def default_hours(self):
        """Get the hours of the parent Employee.
        """
        base = self.base_contract()
        if base is None or base == self:
            return ''
        return base.getHours()

    def default_employment_type(self):
        """Get the employment type of the parent Employee.
        """
        base = self.base_contract()
        if base is None or base == self:
            return ''
        return base.getEmploymentType()

    def default_days_per_week(self):
        """Get the days per week of the parent Employee.
        """
        base = self.base_contract()
        if base is None or base == self:
            return 0
        return base.getDaysPerWeek()

    def expiry_date(self):
        """Expiry date of this contract.

        Basically start date + duration.  But e.g. a contract of one
        month starting at 31 January should end at 28 (or 29) February.

        Will return None if start date or duration is not known.
        """
        start = self.getStartdate()
        duration = self.getDuration()
        if start is None or duration is None:
            return None
        year, month, day = start.year(), start.month(), start.day()
        year = year + (month + duration - 1) / 12
        month = 1 + (month + duration - 1) % 12
        first_of_month = DateTime(year, month, 1)
        date = first_of_month + day - 1
        # 31 January + 31 days is 3 March, so count backwards in that
        # case.  Watch out for infinite loops here...
        safety_count = 5
        while date.month() != month and safety_count > 0:
            safety_count -= 1
            date -= 1
        if not safety_count:
            return None
        return date

    security.declarePublic('hour_spread')
    @property
    def hour_spread(self):
        """ Returns the object ContractHourSpread linked to the contract.
        If the object do not exist yet, then it is created.
        """
        annotations = IAnnotations(self)
        ANNO_KEY = 'plonehrm.contract'

        metadata = annotations.get(ANNO_KEY, None)
        if metadata is None:
            annotations[ANNO_KEY] = PersistentDict()
            metadata = annotations[ANNO_KEY]

        if not 'hour_spread' in metadata:
            metadata['hour_spread'] = ContractHourSpread()

        return metadata['hour_spread']


registerType(Contract, config.PROJECTNAME)



from zope.interface import Interface


class IContract(Interface):

    def getId():
        """ """

    def setId():
        """ """

    def Title():
        """ """

    def Description():
        """ """

    def setDescription():
        """ """

    def getWage():
        """return the current wage"""

    def setWage():
        """set the current wage"""

    def getFunction():
        """Returns the function"""

    def setFunction():
        """Set the function"""

    def getDuration():
        """ """

    def setDuration():
        """ """

    def getEmploymentType():
        """ """

    def setEmploymentType():
        """ """

    def getHoures():
        """ """

    def setHours():
        """ """

    def getDaysPerWeek():
        """ """
        
    def setDaysPerWeek():
        """ """
    
    def getTemplate():
        """ """

    def setTemplate():
        """ """


class ILetter(IContract):
    """An official letter"""


class IContractTool(Interface):
    """Tool to manage templates, positions, and contract types."""
